'**************************************************************************************************
' Program: Subroutine_Fisher_Agg.prg                                                              *
' Purpose: This subroutine applies the Fisher aggregation formula for any frequency.              *
'          For an introduction to the calculation and use of Fisher indexes, particularly in the  *
'          context of National Accounts data, see Karl Whelan, "A Guide to the Use of Chain       *
'          Aggregated NIPA Data" (June 2000), available at                                        *
'          https://www.federalreserve.gov/pubs/feds/2000/200035/200035pap.pdf.                    *
'                                                                                                 *
'	   The subroutine takes the following arguments:                                          *
'	   fisher_agg(GROUP data, SERIES aggregate, STRING frequency,                             *
'                                                     STRING base, STRING %begin, STRING %end)    *
'                                                                                                 *
'	   data:      a group object containing the nominal components being aggregated           *
'	   aggregate: a series containing the nominal sum of the above components                 *
'	   frequency: a string indicating the frequency of the data (denoted A, Q, or M)          *
'	   base:      a string indicating the base year of the data                               *
'	   begin:     a string containing the start date of the aggregation                       *
'	   end:	      a string containing the end date of the aggregation                         *
'                                                                                                 *
'	   NOTE: The page in which the subroutine is called must contain the real and price       *
'                series corresponding to the series in "data".                                    *
'                                                                                                 *
'	   EXAMPLE:                                                                               *
'	   group component_list ccapdenrnfoth ccapdenrnfeqc ccapdenrnfsft ccapdenrnfcomm          *
'	   series ccapdenrnf = ccapdenrnfoth + ccapdenrnfeqc + ccapdenrnfsft + ccapdenrnfcomm     *
'	   call fisher_agg(component_list, ccapdenrnf, @pagefreq, "2009", %begin, %end)           *
' Author: Bob Shackleton                                                                          *
' Date Created: March 30, 2011                                                                    *
'**************************************************************************************************

SUBROUTINE fisher_agg(GROUP data, SERIES aggregate, STRING frequency, STRING base, STRING %begin, STRING %end) 

' Determine data frequency and save the base year
%freq = @upper(frequency)
!freq = 0
!base = @val(base)

IF %freq = "A" THEN
     !freq = 1
ELSE
     IF %freq = "Q" THEN
          !freq = 4
     ELSE
          IF %freq = "M" THEN
               !freq = 12
          ELSE
               @uiprompt("Frequency Problem in Fisher Subroutine")
               STOP
          ENDIF
     ENDIF
ENDIF

' Calculate the Laspeyres, Paasche, and Fisher indices _
' (Paasche & Fisher indices will not be created in the first year

SMPL {%begin} {%end}

SERIES laspeyres_q = 0
SERIES laspeyres_p = 0
SERIES check_weights = 0

SMPL {%begin}+1 {%end}

FOR !x = 1 TO data.@count
     %series_name = data.@seriesname(!x)
     %q_series_name = "Q" + %series_name
     %p_series_name = "P" + %series_name 
     SERIES laspeyres_q   = laspeyres_q + (@pc({%q_series_name}) * _
                            {%series_name}(-1) / aggregate(-1))
     SERIES laspeyres_p   = laspeyres_p + (@pc({%p_series_name}) * _
                            {%series_name}(-1) / aggregate(-1))
     ' This is just to make sure weight sum to 1
     SERIES check_weights = check_weights + _
                            ({%series_name}(-1) / aggregate(-1))  
NEXT

SMPL {%begin} {%end}

SERIES laspeyres_q = 1 + (laspeyres_q / 100)
SERIES laspeyres_p = 1 + (laspeyres_p / 100)

' @uiedit(%stopgo,"OK?")
' if %stopgo = "n" then
'      stop
' endif

SMPL {%begin}+1 {%end}

SERIES paasche_q   = (1 + @pc(aggregate) / 100) / laspeyres_p
SERIES paasche_p   = (1 + @pc(aggregate) / 100) / laspeyres_q
SERIES fisher_q    = (laspeyres_q * paasche_q)^(0.5)
SERIES fisher_p    = (laspeyres_p * paasche_p)^(0.5)

' Create the Price Index

%p_aggregate_name = "P" + aggregate.@name

SMPL {%begin} {%end}

SERIES {%p_aggregate_name} = 1
SERIES moving_average_p = 1

SMPL {%begin}+1 {%end}

SERIES {%p_aggregate_name} = {%p_aggregate_name}(-1) * fisher_p
SERIES moving_average_p = @movav({%p_aggregate_name}, {!freq})

SMPL {%begin} {%end}

IF %freq = "A" THEN
     SERIES {%p_aggregate_name} = {%p_aggregate_name} / _
                                  @elem(moving_average_p, !base)
ELSE
     IF %freq = "Q" OR %freq = "M" THEN
          SERIES {%p_aggregate_name} = {%p_aggregate_name} / _
                               @elem(moving_average_p, !base:{!freq})
     ENDIF
ENDIF

' Create the Quantity in Levels

%q_aggregate_name = "Q" + aggregate.@name

SMPL {%begin} {%end}
SERIES {%q_aggregate_name} = 1
SERIES moving_average_q = 1

SMPL {%begin}+1 {%end}

SERIES {%q_aggregate_name} = {%q_aggregate_name}(-1) * fisher_q
SERIES moving_average_q = @movav(aggregate, {!freq}) / _
                          @movav({%q_aggregate_name}, {!freq})

SMPL {%begin} {%end}

IF %freq = "A" THEN
     SERIES {%q_aggregate_name} = {%q_aggregate_name} * _
                                  @elem(moving_average_q, !base)
ELSE
     IF %freq = "Q" OR %freq = "M" THEN
          SERIES {%q_aggregate_name} = {%q_aggregate_name} * _
                               @elem(moving_average_q, !base:{!freq})
     ENDIF
ENDIF

DELETE laspeyres_p laspeyres_q check_weights paasche_p paasche_q _
       fisher_p fisher_q moving_average_p moving_average_q 

ENDSUB
